@@ -1,5 +1,5 @@
 /* GNU Chess 5.0 - common.h - common symbolic definitions
-   Copyright (c) 1999 Free Software Foundation, Inc.
+   Copyright (c) 1999-2002 Free Software Foundation, Inc.
 
    GNU Chess is based on the two research programs 
    Cobalt by Chua Kong-Sian and Gazebo by Stuart Cracraft.
@@ -33,58 +33,105 @@
 # define __attribute__(x)
 #endif
 
- /*
-  * Include "uint64_t" and similar types using the ac_need_stdint_h ac macro
-  */
+/*
+ * Include "uint64_t" and similar types using the ac_need_stdint_h ac macro
+ */
+
 #include "GCint.h"
 
+#include <stdio.h>
+#include <sys/types.h>
+
+ /* 
+  * Define time structures to get timeval for Timer 
+  */
+
+#include <sys/time.h>
+
+ /*
+  * Define macro for declaring 64bit constants for compilers not using ULL
+  */
+
+#ifdef _MSC_VER
+   #define ULL(x) ((uint64_t)(x))
+#else
+   #define ULL(x) x ## ULL
+#endif
+
+/*
+ * BitBoard is a key data type.  It is a 64-bit value, in which each
+ * bit represents a square on the board as defined by "enum Square".
+ * Thus, bit position 0 represents a fact about square a1, and bit
+ * position 63 represents a fact about square h8.  For example, the
+ * bitboard representing "white rook" positions will have a bit set
+ * for every position occupied on the board by a white rook.
+ */
+
 typedef uint64_t BitBoard;
 typedef uint64_t HashType;
 typedef uint32_t KeyType;
 
+/*
+ * Board represents an entire board's state, and is structured to
+ * simplify analysis:
+ */
+
 typedef struct 
 {
-   BitBoard b[2][7];
-   BitBoard friends[2];
+   BitBoard b[2][7];      /* piece/pawn positions by side (0=white, 1=black)
+                             and then by piece (1=pawn..6=king). For example,
+                             b[white][knight] has bits set for every board
+                             position occupied by a White Knight. */
+   BitBoard friends[2];   /* Friendly (this side's) pieces */
    BitBoard blocker;
-   BitBoard blockerr90;
-   BitBoard blockerr45;
-   BitBoard blockerr315;
-   short ep;
-   short flag;
-   short side;
-   short material[2];
-   short pmaterial[2];
-   short castled[2];
-   short king[2];
+   BitBoard blockerr90;   /* rotated 90 degrees */
+   BitBoard blockerr45;   /* rotated 45 degrees */
+   BitBoard blockerr315;  /* rotated 315 degrees */
+   short ep;              /* Location of en passant square */
+   short flag;            /* Flags related to castle privileges */
+   short side;            /* Color of side on move: 0=white, 1=black */
+   short material[2];     /* Total material by side not inc. king */
+   short pmaterial[2];    /* Total pawn material by side not inc. king */
+   short castled[2];      /* True (1) if side is castled */
+   short king[2];         /* Location of king 0 - a1 .. 63 - h8 */
 } Board; 
 
-typedef struct
-{
-   int move;
-   int score; 
-} leaf;
+/* leaf describes a leaf-level analysis result */
 
 typedef struct
 {
-   int move;
-   short epsq; 
-   short bflag;
-   short Game50;
+   int move;   /* the move that produced this particular board */
+   int score;  /* the scored value of this leaf */
+} leaf;
+
+/*
+ * GameRec records an individual move made in the game; an entire
+ * Game is a set of GameRec's:
+ */
+
+#define SANSZ 8 /* longest move is "exf8=Q+" so 7+1 */
+
+typedef struct
+{
+   int move;    /* The actual move made; this is NOT the move count! */
+   short epsq;  /* en passant square */
+   short bflag; /* Flags for castle privs, see Board.flag */
+   short Game50; /* The last value of GameCnt (a count of half-moves)
+                    that castled, captured, or moved a pawn */
    short mvboard;
-   float et;
+   float et;     /* elapsed time */
    HashType hashkey;
    HashType phashkey;
-   char SANmv[8];
+   char SANmv[SANSZ];  /* The move in SAN notation */
 } GameRec;
 
 typedef struct
 {
-   KeyType key; 
-   int move;
-   int score;
-   short flag;
-   short depth;
+   HashType key;    /* Full 64 bit hash */
+   int move;        /* Best move */
+   int score;       /* Estimated score, may be lower or upper bound */
+   uint8_t flag;    /* Is this alpha, beta, quiescent or exact? */
+   uint8_t depth;   /* Search depth */
 } HashSlot;   
 
 typedef struct
@@ -92,17 +139,29 @@ typedef struct
    KeyType pkey;  
    BitBoard passed;
    BitBoard weaked;
-   short score;
-   short phase;
+   int score;
+   int phase;
 } PawnSlot;
 
 
 /*  MACRO definitions */
 
+#ifndef MAX
 #define MAX(a,b)     ((a) > (b) ? (a) : (b))
+#endif
+#ifndef MIN
 #define MIN(a,b)     ((a) < (b) ? (a) : (b))
-#define SET(a,b)     (a |= b)
-#define CLEAR(a,b)   (a &= ~b)
+#endif
+#define SET(a,b)                     \
+  do {                               \
+    (a) |= (b);                      \
+    dbg_printf("Set   0x%x\n", (b)); \
+  } while (0)
+#define CLEAR(a,b)                   \
+  do {                               \
+    (a) &= ~(b);                     \
+    dbg_printf("Clear 0x%x\n", (b)); \
+  } while (0)
 #define	DRAWSCORE    (computerplays == board.side ? (opprating - myrating) / 4 : (myrating - opprating ) / 4)
 #define MATERIAL     (board.material[side] - board.material[1^side])
 #define PHASE	     (8 - (board.material[white]+board.material[black]) / 1150)
@@ -130,8 +189,8 @@ typedef struct
  * calculation.
  */
 #if defined(__GNUC__) && defined(__PPC__) && __GNUC__ < 3
-#  define SETBIT(b,i)   ((b) |=  ((1ULL<<63)>>(i)))
-#  define CLEARBIT(b,i) ((b) &= ~((1ULL<<63)>>(i)))
+#  define SETBIT(b,i)   ((b) |=  ((ULL(1)<<63)>>(i)))
+#  define CLEARBIT(b,i) ((b) &= ~((ULL(1)<<63)>>(i)))
 #else
 #  define SETBIT(b,i)   ((b) |= BitPosArray[i])
 #  define CLEARBIT(b,i) ((b) &= NotBitPosArray[i])
@@ -141,12 +200,38 @@ typedef struct
 #define ROW(i) ((i) & 7)
 #define trailz(b) (leadz ((b) & ((~b) + 1)))
 
+/* Move Descriptions (moves) are represented internally as integers.
+ * The lowest 6 bits define the destination ("TO") square, and
+ * the next lowest 6 bits define the source ("FROM") square,
+ * using the values defined by "enum Square" (0=a1, 63=h8).
+ * Upper bits are used to identify other move information such as
+ * a promotion (and to what), a capture (and of what),
+ * CASTLING moves, and ENPASSANT moves; see the "constants for
+ * move description" below for more information on these upper bits.
+ */
 #define PROMOTEPIECE(a) ((a >> 12) & 0x0007)
 #define CAPTUREPIECE(a) ((a >> 15) & 0x0007)
 #define TOSQ(a)         ((a) & 0x003F)
 #define FROMSQ(a)       ((a >> 6) & 0x003F)
 #define MOVE(a,b)       (((a) << 6) | (b))
 
+/* constants for move description */
+#define KNIGHTPRM     0x00002000
+#define BISHOPPRM     0x00003000 
+#define ROOKPRM       0x00004000
+#define QUEENPRM      0x00005000
+#define PROMOTION     0x00007000
+#define PAWNCAP       0x00008000
+#define KNIGHTCAP     0x00010000 
+#define BISHOPCAP     0x00018000
+#define ROOKCAP       0x00020000 
+#define QUEENCAP      0x00028000 
+#define CAPTURE       0x00038000 
+#define NULLMOVE      0x00100000 
+#define CASTLING      0x00200000 
+#define ENPASSANT     0x00400000
+#define MOVEMASK      (CASTLING | ENPASSANT | PROMOTION | 0x0FFF)
+
 #define white  0
 #define black  1
 #define false  0
@@ -173,30 +258,14 @@ typedef struct
 #define ValueQ   1100
 #define ValueK   2000
 
-/* constants for move description */
-#define KNIGHTPRM     0x00002000
-#define BISHOPPRM     0x00003000 
-#define ROOKPRM       0x00004000
-#define QUEENPRM      0x00005000
-#define PROMOTION     0x00007000
-#define PAWNCAP       0x00008000
-#define KNIGHTCAP     0x00010000 
-#define BISHOPCAP     0x00018000
-#define ROOKCAP       0x00020000 
-#define QUEENCAP      0x00028000 
-#define CAPTURE       0x00038000 
-#define NULLMOVE      0x00100000 
-#define CASTLING      0x00200000 
-#define ENPASSANT     0x00400000
-#define MOVEMASK      (CASTLING | ENPASSANT | PROMOTION | 0x0FFF)
 
 /*  Some special BitBoards  */
-#define NULLBITBOARD  (0x0000000000000000ULL)
-#define WHITESQUARES  (0x55AA55AA55AA55AAULL)
-#define BLACKSQUARES  (0xAA55AA55AA55AA55ULL)
-#define CENTRESQUARES (0x0000001818000000ULL)
-#define COMPUTERHALF  (0xFFFFFFFF00000000ULL)
-#define OPPONENTHALF  (0x00000000FFFFFFFFULL)
+#define NULLBITBOARD  ( ULL(0x0000000000000000))
+#define WHITESQUARES  ( ULL(0x55AA55AA55AA55AA))
+#define BLACKSQUARES  ( ULL(0xAA55AA55AA55AA55))
+#define CENTRESQUARES ( ULL(0x0000001818000000))
+#define COMPUTERHALF  ( ULL(0xFFFFFFFF00000000))
+#define OPPONENTHALF  ( ULL(0x00000000FFFFFFFF))
 
 /*  Game flags */
 #define QUIT    0x0001
@@ -212,6 +281,9 @@ typedef struct
 #define XBOARD  0x0400
 #define TIMECTL 0x0800
 #define POST    0x1000
+#define PONDER  0x2000 /* We are in pondering (during search) */
+#define HARD    0x4000 /* Pondering is turned on */
+#define ANALYZE 0x8000 /* In ANALYZE mode */
 
 /*  Node types  */ 
 #define PV  0
@@ -282,6 +354,7 @@ extern BitBoard Bishop45Atak[64][256];
 extern BitBoard Bishop315Atak[64][256];
 extern BitBoard pinned;
 extern BitBoard rings[4];
+extern BitBoard boxes[2];
 extern BitBoard stonewall[2];
 extern BitBoard pieces[2];
 extern BitBoard mask_kr_trapped_w[3];
@@ -296,12 +369,15 @@ extern leaf Tree[MAXTREEDEPTH];
 extern leaf *TreePtr[MAXPLYDEPTH];
 extern int RootPV;
 extern GameRec Game[MAXGAMEDEPTH];
-extern short GameCnt;
-extern short computer;
+extern int RealGameCnt;
+extern short RealSide;
+extern int GameCnt;
+extern int computer;
 extern unsigned int flags;
+extern unsigned int preanalyze_flags;
 extern Board board;
-extern short cboard[64];
-extern short Mvboard[64];
+extern int cboard[64];
+extern int Mvboard[64];
 extern HashType hashcode[2][7][64];
 extern HashType ephash[64];
 extern HashType WKCastlehash;
@@ -313,9 +389,9 @@ extern HashType HashKey;
 extern HashType PawnHashKey;
 extern HashSlot *HashTab[2];
 extern PawnSlot *PawnTab[2];
-extern short Idepth;
-extern short SxDec;
-extern short Game50;
+extern int Idepth;
+extern int SxDec;
+extern int Game50;
 extern int lazyscore[2];
 extern int maxposnscore[2];
 extern int rootscore;
@@ -345,35 +421,26 @@ extern unsigned long RepeatCnt;
 extern unsigned HashSize;
 extern unsigned long TTHashMask;
 extern unsigned long PHashMask;
-extern short slider[8];
-extern short Value[7];
-extern char SANmv[10];
+extern int slider[8];
+extern int Value[7];
+extern char SANmv[SANSZ];
 extern unsigned long history[2][4096];
 extern int killer1[MAXPLYDEPTH];
 extern int killer2[MAXPLYDEPTH];
-extern short ChkCnt[MAXPLYDEPTH];
-extern short ThrtCnt[MAXPLYDEPTH];
+extern int ChkCnt[MAXPLYDEPTH];
+extern int ThrtCnt[MAXPLYDEPTH];
 extern char id[32];
 extern char solution[64];
-/*
- * XXX: This variable et also exists as a local variable somewhere and
- * to complete the confusion, this came up in three different
- * flavours: long, float and double.
- */
-extern double et;
 extern float SearchTime;
-extern short SearchDepth;
-extern short MoveLimit[2];
+extern int SearchDepth;
+extern int MoveLimit[2];
 extern float TimeLimit[2];
-extern short TCMove;
-extern short TCinc;
+extern int TCMove;
+extern int TCinc;
 extern float TCTime;
-extern short hunged[2];
-extern short phase;
+extern int hunged[2];
+extern int phase;
 extern int Hashmv[MAXPLYDEPTH];
-extern short DebugPly;
-extern short DebugDepth;
-extern long DebugNode;
 extern int Debugmv[MAXPLYDEPTH];
 extern short Debugmvl;
 extern short Debugline;
@@ -386,36 +453,44 @@ extern short pickphase[MAXPLYDEPTH];
 extern short InChk[MAXPLYDEPTH];
 extern short KingThrt[2][MAXPLYDEPTH];
 extern short threatmv;
-extern short threatply;
+extern uint8_t threatply;
 extern short KingSafety[2];
 extern short pscore[64];
 
 extern short bookmode;
 extern short bookfirstlast;
 
-extern short range[8];
-extern short ptype[2];
+extern int range[8];
+extern int ptype[2];
 extern char algbr[64][3];
 extern char algbrfile[9];
 extern char algbrrank[9];
 extern char notation[8];
 extern char lnotation[8];
-extern short r90[64];
-extern short r45[64];
-extern short r315[64];
-extern short Mask45[64];
-extern short Mask315[64];
+extern int r90[64];
+extern int r45[64];
+extern int r315[64];
+extern int Mask45[64];
+extern int Mask315[64];
+
+extern int rank6[2];
+extern int rank7[2];
+extern int rank8[2];
+
+extern const char *progname;
 extern FILE *ofp;
-extern short myrating, opprating, suddendeath;
-extern char name[50];
-extern short computerplays;
-extern short wasbookmove;
+extern int myrating, opprating, suddendeath;
+
+#define MAXNAMESZ 50
+extern char name[MAXNAMESZ];
+extern int computerplays;
+extern int wasbookmove;
 extern int nmovesfrombook;
 extern float maxtime;
-extern short n; 		/* Last mobility returned by CTL */
-extern short ExchCnt[2];
+extern int n; 		/* Last mobility returned by CTL */
+extern int ExchCnt[2];
 extern int newpos, existpos;		/* For book statistics */
-extern short bookloaded;
+extern int bookloaded;
 
 enum Piece { empty, pawn, knight, bishop, rook, queen, king, bpawn };
 
@@ -466,12 +541,16 @@ void InitHashCode (void);
 void InitHashTable (void);
 void NewPosition (void);
 void InitFICS (void);
+void InitInput (void);
+
+/* Cleanup routines */
+void CleanupInput(void);
 
 /*  The book routines */
 void MakeBinBook (char *, short);
 int BookQuery (int);
 int BookBuilderOpen(void);
-int BookBuilder (short result, short side);
+int BookBuilder (short result, uint8_t side);
 int BookBuilderClose(void);
 
 /*
@@ -497,48 +576,42 @@ void GenCheckEscapes (short);
 void FilterIllegalMoves (short);
 
 /*  The move routines  */
-void MakeMove (short, int *);
-void UnmakeMove (short, int *);
-void MakeNullMove (short);
-void UnmakeNullMove (short);
-void SANMove (int, short);
+void MakeMove (int, int *);
+void UnmakeMove (int, int *);
+void MakeNullMove (int);
+void UnmakeNullMove (int);
+void SANMove (int, int);
 leaf *ValidateMove (char *);
-leaf *IsInMoveList (short, short, short, char);
-short IsLegalMove (int);
+leaf *IsInMoveList (int, int, int, char);
+int IsLegalMove (int);
 char *AlgbrMove (int);
 
 /*  The attack routines  */
-short SqAtakd (short, short);
+short SqAtakd (short sq, short side);
 void GenAtaks (void);
-BitBoard AttackTo (short, short);
-BitBoard AttackXTo (short, short);
-BitBoard AttackFrom (short, short, short);
-BitBoard AttackXFrom (short, short);
-short PinnedOnKing (short, short);
+BitBoard AttackTo (int, int);
+BitBoard AttackXTo (int, int);
+BitBoard AttackFrom (int, int, int);
+BitBoard AttackXFrom (int, int);
+int PinnedOnKing (int, int);
 void FindPins (BitBoard *);
-short MateScan (short);
+int MateScan (int);
 
 /*  The swap routines  */
-short SwapOff (int);
-void AddXrayPiece (short, short, short, BitBoard *, BitBoard *);
+int SwapOff (int);
+void AddXrayPiece (int, int, int, BitBoard *, BitBoard *);
 
 /*  The EPD routines  */
 short ReadEPDFile (const char *, short);
-void ParseEPD (char *);
+int ParseEPD (char *);
 void LoadEPD (char *);
 void SaveEPD (char *);
 
-/*  The command routines */
-void InputCmd (void);
-void ShowCmd (char *);
-void BookCmd (char *);
-void TestCmd (char *);
-
-/*  Some utility routines  */
-#ifdef NO_INLINE
-unsigned char leadz (BitBoard);
-unsigned char nbits (BitBoard);
-#endif
+/* Error codes for ParseEPD */
+enum {
+   EPD_SUCCESS,
+   EPD_ERROR
+};
 
 void UpdateFriends (void);
 void UpdateCBoard (void);
@@ -567,34 +640,47 @@ int Evaluate (int, int);
 short EvaluateDraw (void);
 
 /*  Hung routines  */
-short EvalHung (short);
+int EvalHung (int);
 
 /*  The search routines  */
 void Iterate (void);
-int Search (short, short, int, int, short);
-int SearchRoot (short, int, int);
-int Quiesce (short, int, int);
+int Search (uint8_t ply, short depth, int alpha, int beta, short nodetype);
+int SearchRoot (short depth, int alpha, int beta);
+int Quiesce (uint8_t ply, int alpha, int beta);
 void pick (leaf *, short);
 short Repeat (void);
 void ShowLine (int, int, char);
-void GetElapsed (void);
+
+/*
+ * Set up a timer by first calling StartTiming(), saving
+ * the return value and feeding it to GetElapsed() for
+ * timings in seconds.
+ */
+typedef struct timeval Timer;
+extern double ElapsedTime;
+extern Timer StartTime;
+Timer StartTiming (void);
+double GetElapsed (Timer start);
 
 /*  The transposition table routies */
-void TTPut (short, short, short, int, int, int, int);
-short TTGet (short, short, short, int, int, int *, int *);
-short TTGetPV (short, short, int, int *);
+void TTPut (uint8_t side, uint8_t depth, uint8_t ply, 
+	    int alpha, int beta, int score, int move);
+/* Returns flag if it finds an entry, 0 otherwise */
+uint8_t TTGet (uint8_t side, uint8_t depth, uint8_t ply,
+	     int *score, int *move);
+short TTGetPV (uint8_t side, uint8_t ply, int score, int *move);
 void TTClear (void);
 void PTClear (void);
 
 /*  Sorting routines  */
-void SortCaptures (short);
-void SortMoves (short);
+void SortCaptures (int);
+void SortMoves (int);
 void SortRoot (void);
-short PhasePick (leaf **, short);
-short PhasePick1 (leaf **, short);
+int PhasePick (leaf **, int);
+int PhasePick1 (leaf **, int);
 
 /*  Some output routines */
-void ShowMoveList (short);
+void ShowMoveList (int);
 void ShowSmallBoard (void);
 void ShowBoard (void);
 void ShowBitBoard (BitBoard *);
@@ -605,7 +691,7 @@ void ShowGame (void);
 void ShowTime (void);
 
 /*  Random numbers routines */
-unsigned int Rand32 (void);
+uint32_t Rand32 (void);
 HashType Rand64 (void);
 
 /*  Solver routines  */
@@ -634,8 +720,152 @@ int DBSearchPlayer (const char *player);
 void DBUpdatePlayer (const char *player, const char *resultstr);
 void DBTest (void);
 
-#ifndef NO_INLINE
+/* Input thread and thread function */
+#include <pthread.h>
+extern pthread_t input_thread;
+void *input_func(void *);
+
+/*
+ * Status variable used by the input thread to signal
+ * pending input. Thought about using flags for this
+ * but this can be refined and is conceptually different
+ * from flags.
+ */
+enum {
+  INPUT_NONE,
+  INPUT_AVAILABLE
+};
+extern volatile int input_status;
+
+/*
+ * Function to wake up the input thread, should be called after
+ * input has been parsed.
+ */
+void input_wakeup(void);
+
+/* Wait for input. */
+
+void wait_for_input(void);
+
+/*
+ * Input routine, initialized to one of the specific
+ * input routines. The given argument is the prompt.
+ */
+void (*getline) (char *);
+
+#define MAXSTR 128
+extern char inputstr[MAXSTR];
+
+/* Input parser */
+void parse_input(void);
+
+/* Pondering */
+void ponder(void);
+
+/*  The command subroutines */
+void ShowCmd (char *);
+void BookCmd (char *);
+void TestCmd (char *);
+
+/* Commands from the input engine */
+void cmd_accepted(void);
+void cmd_activate(void); 
+void cmd_analyze(void);
+void cmd_bk(void);
+void cmd_black(void);
+void cmd_book(void);
+void cmd_computer(void);
+void cmd_debug(void);
+void cmd_debugdepth(void);
+void cmd_debugnode(void);
+void cmd_debugply(void);
+void cmd_depth(void);
+void cmd_draw(void);
+void cmd_easy(void);
+void cmd_edit(void); 
+void cmd_epd(void);
+void cmd_exit(void);
+void cmd_force(void);
+void cmd_go(void);
+void cmd_hard(void);
+void cmd_hash(void);
+void cmd_hashsize(void);
+void cmd_help (void);
+void cmd_hint(void);
+void cmd_level(void);
+void cmd_list(void);
+void cmd_load(void);
+void cmd_manual(void);
+void cmd_movenow(void);
+void cmd_name(void);
+void cmd_new(void);
+void cmd_nopost(void);
+void cmd_null(void);
+void cmd_otim(void);
+void cmd_pgnload(void);
+void cmd_pgnsave(void);
+void cmd_ping(void);
+void cmd_post(void);
+void cmd_protover(void);
+void cmd_quit(void);
+void cmd_random(void);
+void cmd_rating(void);
+void cmd_rejected(void);
+void cmd_remove(void);
+void cmd_result(void);
+void cmd_save(void);
+void cmd_setboard(void);
+void cmd_show (void);
+void cmd_solve(void);
+void cmd_st(void); 
+void cmd_switch(void);
+void cmd_test (void);
+void cmd_time(void);
+void cmd_undo(void);
+void cmd_usage(void);
+void cmd_variant(void);
+void cmd_version(void);
+void cmd_white(void);
+void cmd_xboard(void);
+
+/*
+ * Define NO_INLINE only if you really must, implementations will be
+ * provided by the corresponding *.c files. The better solution is to
+ * not define it, in which case inlines.h will be included which
+ * provides static inline version of these functions.
+ */
+
+/*  Some utility routines  */
+#ifdef NO_INLINE
+unsigned char leadz (BitBoard);
+unsigned char nbits (BitBoard);
+#else
 # include "inlines.h"
 #endif
 
+/* More elaborate debugging output to logfile */
+
+/* All the following functions are no-ops if DEBUG is not defined */
+
+/*
+ * dbg_open() can be called with NULL as argument, using a default
+ * filename, defined in debug.c, for the debug log. Otherwise the
+ * argument is the filename. If dbg_open() fails or is not called at
+ * all, debugging output goes to stderr by default.
+ */
+int dbg_open(const char *name);
+
+/* Same format rules as printf() */
+int dbg_printf(const char *fmt, ...);
+
+/* Closes the debugging log, if it is not stderr */
+int dbg_close(void);
+
+# ifdef DEBUG
+#  include <assert.h>
+#  define ASSERT(x) assert(x)
+# else
+#  define ASSERT(x)
+# endif
+
 #endif /* !COMMON_H */
