@@ -1,5 +1,5 @@
 /* GNU Chess 5.0 - eval.c - evaluation code
-   Copyright (c) 1999 Free Software Foundation, Inc.
+   Copyright (c) 1999-2002 Free Software Foundation, Inc.
 
    GNU Chess is based on the two research programs 
    Cobalt by Chua Kong-Sian and Gazebo by Stuart Cracraft.
@@ -37,16 +37,16 @@
 #include "common.h"
 #include "eval.h"
 
-int LoneKing (short, short);
-int ScoreKBNK (short, short);
-int KPK (short);
+int LoneKing (int, int);
+int ScoreKBNK (int, int);
+int KPK (int);
 int BishopTrapped (short);
 int DoubleQR7 (short);
 
 BitBoard passed[2];
 BitBoard weaked[2];
 
-static short PawnSq[2][64] = 
+static int PawnSq[2][64] = 
 {
 {  0,  0,  0,  0,  0,  0,  0,  0,
    5,  5,  5,-10,-10,  5,  5,  5,
@@ -66,29 +66,27 @@ static short PawnSq[2][64] =
    0,  0,  0,  0,  0,  0,  0,  0}
 };
 
-static const short Passed[2][8] =
-{ { 0, 48, 48, 96, 144, 192, 240, 0}, {0, 240, 192, 144, 96, 48, 48, 0} };
+static const int Passed[2][8] =
+{ { 0, 48, 48, 120, 144, 192, 240, 0}, {0, 240, 192, 144, 120, 48, 48, 0} };
 /* Penalties for one or more isolated pawns on a given file */
-static const short isolani_normal[8] = {
+static const int isolani_normal[8] = {
   -8, -10, -12, -14, -14, -12, -10, -8
 };
 /* Penalties if the file is half-open (i.e. no enemy pawns on it) */
-static const short isolani_weaker[8] = {
+static const int isolani_weaker[8] = {
   -22, -24, -26, -28, -28, -26, -24, -22
 };
 
-static const short rank7[2] = { 6, 1 };
-static const short rank8[2] = { 7, 0 };
 static const BitBoard d2e2[2] =
-		 { 0x0018000000000000ULL, 0x0000000000001800ULL };
-static const BitBoard brank7[2]  = { 0x000000000000FF00ULL,
-				     0x00FF000000000000ULL };
-static const BitBoard brank8[2]  = { 0x00000000000000FFULL,
-				     0xFF00000000000000ULL };
-static const BitBoard brank67[2] = { 0x0000000000FFFF00ULL,
-				     0x00FFFF0000000000ULL };
-static const BitBoard brank58[2] = { 0x00000000FFFFFFFFULL,
-				     0xFFFFFFFF00000000ULL };
+		 { ULL(0x0018000000000000), ULL(0x0000000000001800) };
+static const BitBoard brank7[2]  = { ULL(0x000000000000FF00),
+				     ULL(0x00FF000000000000) };
+static const BitBoard brank8[2]  = { ULL(0x00000000000000FF),
+				     ULL(0xFF00000000000000) };
+static const BitBoard brank67[2] = { ULL(0x0000000000FFFF00),
+				     ULL(0x00FFFF0000000000) };
+static const BitBoard brank58[2] = { ULL(0x00000000FFFFFFFF),
+				     ULL(0xFFFFFFFF00000000) };
 
 int ScoreP (short side)
 /***************************************************************************
@@ -109,10 +107,11 @@ int ScoreP (short side)
  *
  ***************************************************************************/
 {
-   short s, sq, i, i1, xside;
-   short n1, n2, backward;
-   short nfile[8];
-   short EnemyKing;
+   int xside;
+   int s, sq, i, i1;
+   int n1, n2, backward;
+   int nfile[8];
+   int EnemyKing;
    BitBoard c, t, p, blocker, *e;
    PawnSlot *ptable;
 
@@ -147,15 +146,11 @@ int ScoreP (short side)
       if ((p & PassedPawnMask[side][sq]) == NULLBITBOARD)
       {
 	 if ((side == white && (FromToRay[sq][sq|56] & c) == 0) ||
-	     (side == black && (FromToRay[sq][sq&7] & c) == 0))
-	 {
+	     (side == black && (FromToRay[sq][sq&7] & c) == 0)) 
+         {
             passed[side] |= BitPosArray[sq];
-            if (side == white) {
-                  s += (Passed[side][RANK(sq)] * phase) / 12;
-            } else if (side == black) {
-                  s += (Passed[side][RANK(sq)] * phase) / 12;
-	    }
-	 }
+            s += (Passed[side][RANK(sq)] * phase) / 12;
+         }
       }
 
       /*  Backward pawns */
@@ -233,10 +228,10 @@ int ScoreP (short side)
     n = 0;
     if (side == white)
       n = nbits((c >> 8) & board.b[xside][pawn] &
-               (rings[0]|rings[1]|rings[2]));
+               boxes[1]);
     else
       n = nbits((c << 8) & board.b[xside][pawn] &
-               (rings[0]|rings[1]|rings[2]));
+               boxes[1]);
     if (n > 1)
       s += n * LOCKEDPAWNS;
   }
@@ -339,7 +334,7 @@ phase2:
    return (s);
 }
 
-static const short Outpost[2][64] =
+static const int Outpost[2][64] =
 {
   { 0, 0, 0, 0, 0, 0, 0, 0,
     0, 0, 0, 0, 0, 0, 0, 0,
@@ -370,7 +365,7 @@ static inline int CTL(short sq, short piece __attribute__ ((unused)), short side
  *
  ***************************************************************************/
 {
-  short s, n, EnemyKing, FriendlyKing;
+  int s, n, EnemyKing, FriendlyKing;
   BitBoard controlled;
 
   s = 0;
@@ -381,7 +376,7 @@ static inline int CTL(short sq, short piece __attribute__ ((unused)), short side
   controlled = AttackXFrom (sq, side);
 
   /* Center control */
-  n = nbits (controlled & (rings[0]|rings[1]));
+  n = nbits (controlled & boxes[0]);
   s += 4*n;
 
   /* Attacks against enemy king */
@@ -409,8 +404,9 @@ int ScoreN (short side)
  *
  ***************************************************************************/
 {
-   short s, s1, sq, xside;
-   short EnemyKing;
+   int xside;
+   int s, s1, sq;
+   int EnemyKing;
    BitBoard c, t;
 
    if (board.b[side][knight] == NULLBITBOARD)
@@ -421,6 +417,11 @@ int ScoreN (short side)
    t = board.b[xside][pawn]; 
    EnemyKing = board.king[xside];
 
+   if ( c & pinned )
+   {
+	s += PINNEDKNIGHT * nbits(c & pinned);
+   }
+
    while (c)
    {
       sq = leadz (c);
@@ -429,8 +430,7 @@ int ScoreN (short side)
       /* Control */
       s1 = CTL(sq,knight,side);
   
-      if ( (BitPosArray[sq] & 
-	    (FileBit[0]|FileBit[7]|RankBit[0]|RankBit[7])) != NULLBITBOARD)
+      if ( (BitPosArray[sq] & rings[3]) != NULLBITBOARD)
 	s1 += KNIGHTONRIM;
 
       if (Outpost[side][sq] && 
@@ -447,7 +447,6 @@ int ScoreN (short side)
       if (MoveArray[knight][sq] & weaked[xside])
          s1 += ATAKWEAKPAWN;
 
-      pscore[sq] = s1;
       s += s1;
    }
 
@@ -466,7 +465,8 @@ int ScoreB (short side)
  *
  ****************************************************************************/
 {
-   short s, s1, xside, n, sq, EnemyKing;
+   int xside;
+   int s, s1, n, sq, EnemyKing;
    BitBoard c, t;
 
    if (board.b[side][bishop] == NULLBITBOARD)
@@ -478,6 +478,11 @@ int ScoreB (short side)
    n = 0;
    t = board.b[xside][pawn];
 
+   if ( c & pinned )
+   {
+	s += PINNEDBISHOP * nbits(c & pinned);
+   }
+
    while (c)
    {
       sq = leadz (c);
@@ -489,7 +494,7 @@ int ScoreB (short side)
 
       /*  Outpost bishop */
       if (Outpost[side][sq] && 
-	  !(t & IsolaniMask[ROW (sq)] & PassedPawnMask[side][sq]))
+	  !(t & IsolaniMask[ROW(sq)] & PassedPawnMask[side][sq]))
       {
          s1 += OUTPOSTBISHOP;
 
@@ -518,7 +523,6 @@ int ScoreB (short side)
       if (BishopAttack(sq) & weaked[xside])
          s1 += ATAKWEAKPAWN;
 
-      pscore[sq] = s1;
       s += s1;
    }
 
@@ -540,6 +544,10 @@ int BishopTrapped (short side)
 {
    int s = 0;
 
+   /* Don't waste time */
+   if (board.b[side][bishop] == NULLBITBOARD)
+     return (0); 
+
    if (side == white)
    {
       if ((board.b[white][bishop] & BitPosArray[A7]) &&
@@ -571,7 +579,7 @@ int ScoreR (short side)
  *
  ****************************************************************************/
 {
-   short s, s1, sq, xside, fyle, EnemyKing;
+   int s, s1, sq, xside, fyle, EnemyKing;
    BitBoard c;
 
    if (board.b[side][rook] == NULLBITBOARD)
@@ -581,6 +589,11 @@ int ScoreR (short side)
    xside = side ^ 1;
    EnemyKing = board.king[xside];
 
+   if ( c & pinned )
+   {
+	s += PINNEDROOK * nbits(c & pinned);
+   }
+
    while (c)
    {
       sq = leadz (c);
@@ -626,7 +639,6 @@ int ScoreR (short side)
 	  board.b[xside][pawn] & RankBit[RANK(sq)]))
          s1 += ROOK7RANK;
 
-      pscore[sq] = s1;
       s += s1;
    }
 
@@ -642,7 +654,7 @@ int DoubleQR7 (short side)
  *
  ***************************************************************************/
 {
-   short xside;
+   int xside;
 
    xside = 1^side;
    if (nbits ((board.b[side][queen]|board.b[side][rook]) & brank7[side]) > 1
@@ -663,20 +675,28 @@ int ScoreQ (short side)
  *
  ***************************************************************************/
 {
-   short s, s1, sq, xside, EnemyKing;
+   int xside;
+   int s, s1, sq, EnemyKing;
    BitBoard c;
    
-   if (board.b[side][queen] == NULLBITBOARD)
-      return (0);
    s = s1 = 0;
+
+   /* Try to keep our queen on the board for attacking purposes. */
+   if (board.b[side][queen] == NULLBITBOARD) {
+       if (side == computer) {
+         s += QUEEN_NOT_PRESENT;
+       }
+       return(s);
+    }                                                                           
+
    xside = 1 ^ side;
    c = board.b[side][queen];
    EnemyKing = board.king[xside];
 
-   /* Try to keep our queen on the board for attacking purposes. */
-   if (side == computer)
-     if (c == NULLBITBOARD)
-        s += QUEEN_NOT_PRESENT;
+   if ( c & pinned )
+   {
+	s += PINNEDQUEEN * nbits(c & pinned);
+   }
 
    while (c)
    {
@@ -693,7 +713,6 @@ int ScoreQ (short side)
       if (QueenAttack(sq) & weaked[xside])
          s1 += ATAKWEAKPAWN;
 
-      pscore[sq] = s1;
       s += s1;
    }
 
@@ -701,7 +720,7 @@ int ScoreQ (short side)
 }
 
 
-static const short KingSq[64] =
+static const int KingSq[64] =
 {
    24, 24, 24, 16, 16,  0, 32, 32,
    24, 20, 16, 12, 12, 16, 20, 24,
@@ -713,7 +732,7 @@ static const short KingSq[64] =
    24, 24, 24, 16, 16,  0, 32, 32
 };
 
-static const short EndingKing[64] =
+static const int EndingKing[64] =
 {
    0,  6, 12, 18, 18, 12,  6,  0,
    6, 12, 18, 24, 24, 18, 12,  6,
@@ -725,15 +744,13 @@ static const short EndingKing[64] =
    0,  6, 12, 18, 18, 12,  6,  0
 };
 
-static short pawncover[9] = { -60, -30, 0, 5, 30, 30, 30, 30, 30 };
-static const short factor[9] = { 7, 8, 8, 7, 6, 5, 4, 2, 0, };
-static const BitBoard Corner[2] = { 0xC3C3000000000000ULL,
-				    0x000000000000C3C3ULL };
+static const int pawncover[9] = { -60, -30, 0, 5, 30, 30, 30, 30, 30 };
+static const int factor[9] = { 7, 8, 8, 7, 6, 5, 4, 2, 0, };
 
 int ScoreK (short side)
 /***************************************************************************
  *
- *  1.  king in the corner.
+ *  1.  king in the corner. ?? SRW 2002-08-02 Unclear if implemented
  *  2.  pawns around king.
  *  3.  king on open file.
  *  4.  Uncastled king.
@@ -742,7 +759,8 @@ int ScoreK (short side)
  *
  ***************************************************************************/
 {
-   short s, sq, sq1, n, n1, n2, xside, file, fsq, rank;
+   int xside;
+   int s, sq, sq1, n, n1, n2, file, fsq, rank;
    BitBoard b, x;
 
    s = 0;
@@ -796,7 +814,7 @@ int ScoreK (short side)
 			      board.b[side][pawn] & RankBit[rank-1]);
           }
 	}
-        pawncover[3] = 30;
+        
 	/* Penalize breaking the wing pawn formations prior to castle */
 	if (n != -1) s += pawncover[n];
       }
@@ -969,12 +987,11 @@ int ScoreK (short side)
       }
    }
 
-   pscore[sq] = s;
    return (s);
 }
 
 
-int LoneKing (short side, short loser)
+int LoneKing (int side, int loser)
 /**************************************************************************
  *
  *  One side has a lonely king and the other has no pawns, but enough
@@ -983,7 +1000,7 @@ int LoneKing (short side, short loser)
  *
  **************************************************************************/
 {
-   short s, winer, sq1, sq2;
+   int s, winer, sq1, sq2;
 
    winer = 1^loser;
    if (board.material[winer] == ValueB+ValueN && 
@@ -1002,7 +1019,7 @@ int LoneKing (short side, short loser)
 }
 
 
-int KPK (short side)
+int KPK (int side)
 /**************************************************************************
  *
  *  A KPK endgame evaluator.  Side is the one on the move.
@@ -1012,7 +1029,7 @@ int KPK (short side)
  *
  **************************************************************************/
 {
-   short winer, loser, sq, sqw, sql;
+   int winer, loser, sq, sqw, sql;
    int s;
 
    winer = (board.b[white][pawn] ? white : black);
@@ -1139,7 +1156,7 @@ int KPK (short side)
 }
 
 
-short KBNK[64] = 
+int KBNK[64] = 
 {
    0, 10, 20, 30, 40, 50, 60, 70,
   10, 20, 30, 40, 50, 60, 70, 60,
@@ -1151,14 +1168,14 @@ short KBNK[64] =
   70, 60, 50, 40, 30, 20, 10,  0
 };
 
-int ScoreKBNK (short side, short loser)
+int ScoreKBNK (int side, int loser)
 /****************************************************************************
  *
  *  My very own KBNK routine!
  *
  ****************************************************************************/
 {
-   short s, winer, sq1, sq2, sqB;
+   int s, winer, sq1, sq2, sqB;
 
    winer = 1^loser;
    sqB = board.king[loser];
@@ -1173,7 +1190,7 @@ int ScoreKBNK (short side, short loser)
    s -= taxicab[leadz(board.b[winer][bishop])][sq2];
 
    /*  King in the central 4x4 region is good! */
-   if (board.b[winer][king] & 0x00003C3C3C3C0000ULL)
+   if (board.b[winer][king] & ULL(0x00003C3C3C3C0000))
       s += 20;
    if (side == loser)
       s = -s;
@@ -1183,10 +1200,8 @@ int ScoreKBNK (short side, short loser)
 }
 
 
-static const BitBoard nn[2] = { 0x4200000000000000ULL, 0x0000000000000042ULL };
-static const BitBoard bb[2] = { 0x2400000000000000ULL, 0x0000000000000024ULL };
-static const BitBoard rr[2] = { 0x8100000000000000ULL, 0x0000000000000081ULL };
-static const BitBoard corner = 0xC3C300000000C3C3ULL;
+static const BitBoard nn[2] = { ULL(0x4200000000000000), ULL(0x0000000000000042) };
+static const BitBoard bb[2] = { ULL(0x2400000000000000), ULL(0x0000000000000024) };
 
 int ScoreDev (short side)
 /***************************************************************************
@@ -1200,7 +1215,8 @@ int ScoreDev (short side)
  ***************************************************************************/
 {
    int s = 0;
-   short sq, xside;
+   int xside;
+   int sq;
    BitBoard c;
 
    xside = 1 ^ side;
@@ -1288,8 +1304,9 @@ int Evaluate (int alpha, int beta)
  *
  ****************************************************************************/
 {
-   short side, xside, piece, s, s1, score;
-   short npiece[2];
+   int side, xside;
+   int piece, s, s1, score;
+   int npiece[2];
    BitBoard *b;
 
    side = board.side;
@@ -1326,9 +1343,8 @@ int Evaluate (int alpha, int beta)
    b = board.b[black];
    pieces[black] = b[knight] | b[bishop] | b[rook] | b[queen];
    npiece[black] = nbits (pieces[black]);
-   hunged[white] = hunged[black] = 0;
-   memset (pscore, 0, sizeof (pscore));
    s1 = MATERIAL;
+
    if ((s1 + maxposnscore[side] < alpha || s1 - maxposnscore[xside] > beta) &&
 	phase <= 6)
    {
@@ -1349,12 +1365,19 @@ int Evaluate (int alpha, int beta)
  *  See if we can have a lazy evaluation cut.  Otherwise its a slow eval.
  * 
  **************************************************************************/
+
    if (s1 + lazyscore[side] < alpha || s1 - lazyscore[side] > beta)
+   {
       score = s1;
+   }
    else
    {
       EvalCnt++;
-      s1 = 0;
+
+	GenAtaks();
+	s1 = HUNGPENALTY * ( EvalHung(side) - EvalHung(xside) );
+	FindPins(&pinned);
+
       for (piece = knight; piece < king; piece++)
       {
          s1 += (*ScorePiece[piece]) (side) - (*ScorePiece[piece]) (xside);
@@ -1412,7 +1435,7 @@ next:
 }
 
 
-short EvaluateDraw ()
+short EvaluateDraw (void)
 /***************************************************************************
  *
  *  This routine is called by search() and quiesce() before anything else
@@ -1426,7 +1449,20 @@ short EvaluateDraw ()
  ***************************************************************************/
 {
    BitBoard *w, *b;
-   short wm, bm, wn, bn;
+   int wm, bm, wn, bn;
+
+   /* 
+    * Exception - if we are close to a pawn move, promotion 
+    * or capture it is possible a forced mate will follow.
+    * So we assume not drawn for 2 moves.
+    */
+
+   if ( (GameCnt-Game50) < 5 )
+     return (false);
+
+   /* 50 move rule */
+   if ( (GameCnt-Game50) > 100 )
+     return (true);
 
    w = board.b[white];
    b = board.b[black];
