@@ -1,5 +1,5 @@
 /* GNU Chess 5.0 - init.c - initialization of variables code
-   Copyright (c) 1999 Free Software Foundation, Inc.
+   Copyright (c) 1999-2002 Free Software Foundation, Inc.
 
    GNU Chess is based on the two research programs 
    Cobalt by Chua Kong-Sian and Gazebo by Stuart Cracraft.
@@ -23,24 +23,17 @@
      bug-gnu-chess@gnu.org
      cracraft@ai.mit.edu, cracraft@stanfordalumni.org, cracraft@earthlink.net
 */
-/*
- *
- */
 
 #include <stdio.h>
 #include <stdlib.h>
 #include <string.h>
 #include <signal.h>
+#include <unistd.h>
+
 #include "common.h"
 #include "version.h"
 
-#ifdef HAVE_READLINE_H
-#include <history.h>
-#elif HAVE_READLINE_READLINE_H
-#include <readline/history.h>
-#endif
-
-void Initialize ()
+void Initialize (void)
 /**************************************************************************
  *
  *  The main initialization driver.
@@ -59,20 +52,16 @@ void Initialize ()
    InitBitCount ();
    InitRotAtak ();
    InitRandomMasks ();
-
    InitDistance ();
    InitVars ();
    InitHashCode ();
    InitHashTable ();
    CalcHashKey ();
-
-#ifdef HAVE_LIBREADLINE
-   using_history();
-#endif
+   InitInput ();
 }
 
 
-void InitFICS ()
+void InitFICS (void)
 {
    if (flags & XBOARD) {
      printf ("tellics shout Greetings from %s %s. Ready for a game.\n", PROGRAM, VERSION);
@@ -83,7 +72,7 @@ void InitFICS ()
 
 #define NBITS 16
 
-void InitLzArray ()
+void InitLzArray (void)
 /***************************************************************************
  *
  *  The lzArray is created.  This array is used when the position
@@ -106,7 +95,7 @@ void InitLzArray ()
 }
 
 
-void InitBitPosArray ()
+void InitBitPosArray (void)
 /***************************************************************************
  *
  *  BitPosArray[i] returns the bitboard whose ith bit is set to 1 
@@ -117,7 +106,7 @@ void InitBitPosArray ()
  ***************************************************************************/
 {
    BitBoard b;
-   short i;
+   int i;
 
    b = (BitBoard) 1;  
    for (i = 63; i >= 0; i--, b <<= 1)
@@ -131,7 +120,7 @@ void InitBitPosArray ()
 
 /*  Data used for generating MoveArray  */
 
-static const short dir[8][8] =
+static const int dir[8][8] =
 {
   { 0, 0, 0, 0, 0, 0, 0, 0 },
   { 9, 11, 0, 0, 0, 0, 0, 0 },
@@ -142,10 +131,10 @@ static const short dir[8][8] =
   { -11, -10, -9, -1, 1, 9, 10, 11 },
   { -9, -11, 0, 0, 0, 0, 0, 0 }
 };
-static const short ndir[8] = 
+static const int ndir[8] = 
 { 0, 2, 8, 4, 4, 8, 8, 2 };
 
-static const short map[120] =
+static const int map[120] =
 {
   -1, -1, -1, -1, -1, -1, -1, -1, -1, -1,
   -1, -1, -1, -1, -1, -1, -1, -1, -1, -1,
@@ -162,7 +151,7 @@ static const short map[120] =
 };
 
 
-void InitMoveArray ()
+void InitMoveArray (void)
 /***************************************************************************
  * 
  *  Generate the move bitboards.  For e.g. the bitboard for all
@@ -170,7 +159,7 @@ void InitMoveArray ()
  *
  ***************************************************************************/
 {
-   short piece, fsq, tsq, f, t, n;
+   int piece, fsq, tsq, f, t, n;
    BitBoard *b;
 
    for (piece = pawn; piece <= bpawn; piece++)
@@ -195,7 +184,7 @@ void InitMoveArray ()
 }
 
 
-void InitRay ()
+void InitRay (void)
 /**************************************************************************
  *
  *  For each square, there are 8 rays.  The first 4 rays are diagonals 
@@ -207,7 +196,7 @@ void InitRay ()
  *
  **************************************************************************/
 {
-   short piece, fsq, tsq, f, t, n, ray;
+   int piece, fsq, tsq, f, t, n, ray;
    BitBoard *b;
 
    memset (directions, -1, sizeof (directions));
@@ -237,7 +226,7 @@ void InitRay ()
 }
 
 
-void InitFromToRay ()
+void InitFromToRay (void)
 /***************************************************************************
  *
  *  The FromToRay[b2][f6] gives the diagonal ray from c3 to f6;
@@ -246,7 +235,7 @@ void InitFromToRay ()
  *
  ***************************************************************************/
 {
-   short piece, fsq, tsq, f, t, n;
+   int piece, fsq, tsq, f, t, n;
    BitBoard *b;
 
    memset (FromToRay, 0, sizeof (FromToRay));
@@ -275,7 +264,7 @@ void InitFromToRay ()
 }
 
 
-void InitRankFileBit ()
+void InitRankFileBit (void)
 /***************************************************************************
  *
  *  RankBit[2] has all the bits on the 3rd rank 1 and others 0.
@@ -284,7 +273,7 @@ void InitRankFileBit ()
  ***************************************************************************/
 {
    BitBoard b;
-   short i;
+   int i;
 
    i = 8;
    b = (BitBoard) 255;
@@ -295,7 +284,7 @@ void InitRankFileBit ()
    }
    
    i = 8;
-   b = 0x0101010101010101ULL;
+   b = ULL(0x0101010101010101);
    while (i--)
    {
       FileBit[i] = b;
@@ -304,7 +293,7 @@ void InitRankFileBit ()
 }
 
 
-void InitRandomMasks ()
+void InitRandomMasks (void)
 {
   mask_kr_trapped_w[0]=BitPosArray[H2];
   mask_kr_trapped_w[1]=BitPosArray[H1]|BitPosArray[H2];
@@ -320,7 +309,7 @@ void InitRandomMasks ()
   mask_qr_trapped_b[2]=BitPosArray[A8]|BitPosArray[B8]|BitPosArray[A7];
 }
 
-void InitPassedPawnMask ()
+void InitPassedPawnMask (void)
 /**************************************************************************
  *
  *  The PassedPawnMask variable is used to determine if a pawn is passed.
@@ -331,7 +320,7 @@ void InitPassedPawnMask ()
  *
  **************************************************************************/
 {
-   unsigned short sq;
+   unsigned int sq;
 
    memset (PassedPawnMask, 0, sizeof (PassedPawnMask));
 
@@ -357,7 +346,7 @@ void InitPassedPawnMask ()
 }
 
 
-void InitIsolaniMask ()
+void InitIsolaniMask (void)
 /**************************************************************************
  *
  *  The IsolaniMask variable is used to determine if a pawn is an isolani.
@@ -369,7 +358,7 @@ void InitIsolaniMask ()
  *
  **************************************************************************/
 {
-   short i;
+   int i;
 
    IsolaniMask[0] = FileBit[1];
    IsolaniMask[7] = FileBit[6];
@@ -379,7 +368,7 @@ void InitIsolaniMask ()
 }
 
 
-void InitSquarePawnMask ()
+void InitSquarePawnMask (void)
 /**************************************************************************
  *
  *  The SquarePawnMask is used to determine if a king is in the square of
@@ -389,8 +378,8 @@ void InitSquarePawnMask ()
  *
  **************************************************************************/
 {
-   unsigned short sq;
-   short len, i, j;
+   unsigned int sq;
+   int len, i, j;
 
    memset (SquarePawnMask, 0, sizeof (PassedPawnMask));
    for (sq = 0; sq < 64; sq++)
@@ -424,7 +413,7 @@ void InitSquarePawnMask ()
 }
 
  
-void InitBitCount ()
+void InitBitCount (void)
 /**************************************************************************
  *
  *  The BitCount array returns the no. of bits present in the 16 bit
@@ -447,7 +436,7 @@ void InitBitCount ()
 } 
 
 
-void InitRotAtak ()
+void InitRotAtak (void)
 /**************************************************************************
  *
  *  The attack tables for a normal chessboard and the rotated chess board
@@ -455,11 +444,11 @@ void InitRotAtak ()
  *
  **************************************************************************/
 {
-   short sq, map, sq1, sq2;
-   short cmap[8] = { 128, 64, 32, 16, 8, 4, 2, 1 };
-   short rot1[8] = { A1, A2, A3, A4, A5, A6, A7, A8 };
-   short rot2[8] = { A1, B2, C3, D4, E5, F6, G7, H8 };
-   short rot3[8] = { A8, B7, C6, D5, E4, F3, G2, H1 };
+   int sq, map, sq1, sq2;
+   int cmap[8] = { 128, 64, 32, 16, 8, 4, 2, 1 };
+   int rot1[8] = { A1, A2, A3, A4, A5, A6, A7, A8 };
+   int rot2[8] = { A1, B2, C3, D4, E5, F6, G7, H8 };
+   int rot3[8] = { A8, B7, C6, D5, E4, F3, G2, H1 };
    
    for (sq = A1; sq <= H1; sq++)
    {
@@ -545,7 +534,7 @@ void InitRotAtak ()
 }
 
 
-void InitDistance ()
+void InitDistance (void)
 /**************************************************************************
  *
  *  There are two arrays dealing with distances.  The distance[s1][s2]
@@ -555,12 +544,12 @@ void InitDistance ()
  *
  *************************************************************************/
 {
-   register short f, t, j;
-   short d1, d2;
+   int f, t, j;
+   int d1, d2;
 
    for (f = 0; f < 64; f++)
      for (t = 0; t < 8; t++)
-       DistMap[f][t] = 0ull;
+       DistMap[f][t] = ULL(0);
 
 
    for (f = 0; f < 64; f++)
@@ -587,7 +576,7 @@ void InitDistance ()
 }
 
 
-void InitVars ()
+void InitVars (void)
 /***************************************************************************
  *
  *  Initialize various variables, especially for new game.
@@ -627,70 +616,13 @@ void InitVars ()
    SETBIT (stonewall[black], E6); /* SMC */
    SETBIT (stonewall[black], F5); /* SMC */
 
-   SETBIT (rings[0], D4);
-   SETBIT (rings[0], D5);
-   SETBIT (rings[0], E4);
-   SETBIT (rings[0], E5);
-   SETBIT (rings[1], C3);
-   SETBIT (rings[1], D3);
-   SETBIT (rings[1], E3);
-   SETBIT (rings[1], F3);
-   SETBIT (rings[1], C4);
-   SETBIT (rings[1], F4);
-   SETBIT (rings[1], C5);
-   SETBIT (rings[1], F5);
-   SETBIT (rings[1], C6);
-   SETBIT (rings[1], D6);
-   SETBIT (rings[1], E6);
-   SETBIT (rings[1], F6);
-   SETBIT (rings[2], B2);
-   SETBIT (rings[2], C2);
-   SETBIT (rings[2], D2);
-   SETBIT (rings[2], E2);
-   SETBIT (rings[2], F2);
-   SETBIT (rings[2], G2);
-   SETBIT (rings[2], B3);
-   SETBIT (rings[2], G3);
-   SETBIT (rings[2], B4);
-   SETBIT (rings[2], G4);
-   SETBIT (rings[2], B5);
-   SETBIT (rings[2], G5);
-   SETBIT (rings[2], B6);
-   SETBIT (rings[2], G6);
-   SETBIT (rings[2], B7);
-   SETBIT (rings[2], C7);
-   SETBIT (rings[2], D7);
-   SETBIT (rings[2], E7);
-   SETBIT (rings[2], F7);
-   SETBIT (rings[2], G7);
-   SETBIT (rings[3], A1);
-   SETBIT (rings[3], B1);
-   SETBIT (rings[3], C1);
-   SETBIT (rings[3], D1);
-   SETBIT (rings[3], E1);
-   SETBIT (rings[3], F1);
-   SETBIT (rings[3], G1);
-   SETBIT (rings[3], H1);
-   SETBIT (rings[3], A2);
-   SETBIT (rings[3], H2);
-   SETBIT (rings[3], A3);
-   SETBIT (rings[3], H3);
-   SETBIT (rings[3], A4);
-   SETBIT (rings[3], H4);
-   SETBIT (rings[3], A5);
-   SETBIT (rings[3], H5);
-   SETBIT (rings[3], A6);
-   SETBIT (rings[3], H6);
-   SETBIT (rings[3], A7);
-   SETBIT (rings[3], H7);
-   SETBIT (rings[3], A8);
-   SETBIT (rings[3], B8);
-   SETBIT (rings[3], C8);
-   SETBIT (rings[3], D8);
-   SETBIT (rings[3], E8);
-   SETBIT (rings[3], F8);
-   SETBIT (rings[3], G8);
-   SETBIT (rings[3], H8);
+   rings[0] = ULL(0x0000001818000000);
+   rings[1] = ULL(0x00003C24243C0000);
+   rings[2] = ULL(0x007E424242427E00);
+   rings[3] = ULL(0xFF818181818181FF);
+
+   boxes[0] = ULL(0x00003C3C3C3C0000); /* rings[0] | rings[1] */
+   boxes[1] = ULL(0x007E7E7E7E7E7E00); /* rings[0] | rings[1] | rings[2] */
 
    boardhalf[white] = RankBit[0]|RankBit[1]|RankBit[2]|RankBit[3];
    boardhalf[black] = RankBit[4]|RankBit[5]|RankBit[6]|RankBit[7];
@@ -698,11 +630,11 @@ void InitVars ()
    boardside[qs] = FileBit[0]|FileBit[1]|FileBit[2]|FileBit[3];
 
    board.flag |= (WCASTLE | BCASTLE);
-   board.side = white;
+   RealSide = board.side = white;
    board.ep = -1;
    board.king[white] = E1;
    board.king[black] = E8;
-   GameCnt = -1;
+   RealGameCnt = GameCnt = -1;
    Game50 = 0;
    computer = black;
    CalcHashKey ();
@@ -756,16 +688,13 @@ void InitVars ()
       PHashMask |= 1;
    }
 
-   DebugPly = 99;
-   DebugDepth = 0;
-   DebugNode = 999999999;
    signal (SIGINT, EndSearch);
 
    nmovesfrombook = 0;
 }
 
 
-void InitHashCode ()
+void InitHashCode (void)
 /***************************************************************************
  *
  *  The hash code for the various pieces standing on the various squares
@@ -775,7 +704,7 @@ void InitHashCode ()
  *
  ***************************************************************************/
 {
-   short color, piece, sq;
+   int color, piece, sq;
 
    for (color = white; color <= black; color++)
    {
@@ -797,7 +726,7 @@ void InitHashCode ()
 }
 
 
-void InitHashTable ()
+void InitHashTable (void)
 /****************************************************************************
  *
  *  Allocate memory for our transposition tables.  By default, the
@@ -808,35 +737,48 @@ void InitHashTable ()
 {
    unsigned int size;
  
-   HashTab[0] = (HashSlot *) realloc (HashTab[0], HashSize * sizeof (HashSlot));
-   HashTab[1] = (HashSlot *) realloc (HashTab[1], HashSize * sizeof (HashSlot));
-//   HashTab[1] = (HashSlot *) realloc (HashSize, sizeof (HashSlot));
-   if (HashTab[0] == NULL || HashTab[1] == NULL)
-      printf ("Not enough memory for transposition table\n");
-   else
-   {
-      size = (HashSize * 2 * sizeof (HashSlot)) >> 10;
-      if (!(flags & XBOARD))
-	printf ("Transposition table:  Entries=%dK Size=%dK\n", 
-		HashSize>>10, size);
+   /* One can not use realloc() here, it screws up error handling */
+   do {
+     free(HashTab[0]);
+     free(HashTab[1]);
+     HashTab[0] = malloc (HashSize * sizeof (HashSlot));
+     HashTab[1] = malloc (HashSize * sizeof (HashSlot));
+     if (HashTab[0] == NULL || HashTab[1] == NULL) {
+        printf ("Not enough memory for transposition table, trying again.\n");
+        HashSize >>= 1;
+	TTHashMask >>= 1;
+	if (HashSize == 0) {
+	  fprintf(stderr, "Memory exhausted, goodbye, my friend.\n");
+	  exit(EXIT_FAILURE);
+	}
+	continue;
+     }
+   } while (0);
+   size = (HashSize * 2 * sizeof (HashSlot)) >> 10;
+   if (!(flags & XBOARD)) {
+     printf ("Transposition table:  Entries=%dK Size=%dK\n", 
+             HashSize>>10, size);
    }
 
+   /*
+    * Here realloc() is O.K., though not very useful, because PAWNSLOTS
+    * is constant anyway.
+    */
    PawnTab[0] = (PawnSlot *) realloc (PawnTab[0], PAWNSLOTS * sizeof (PawnSlot));
    PawnTab[1] = (PawnSlot *) realloc (PawnTab[1], PAWNSLOTS * sizeof (PawnSlot));
-   if (PawnTab[0] == NULL || PawnTab[1] == NULL)
-      printf ("Not enough memory for pawn table\n");
-   else
-   {
+   if (PawnTab[0] == NULL || PawnTab[1] == NULL) {
+      printf ("Not enough memory for pawn table, goodbye.\n");
+      exit(EXIT_FAILURE);
+   } else {
       size = (PAWNSLOTS * 2 * sizeof (PawnSlot)) >> 10;
       if (!(flags & XBOARD))
 	printf ("Pawn hash table: Entries=%dK Size=%dK\n",
 		PAWNSLOTS >> 10, size);
    }
-   return;
 }
 
 
-void NewPosition ()
+void NewPosition (void)
 /****************************************************************************
  *
  *  Reset some flags for a new game/position.
@@ -845,7 +787,7 @@ void NewPosition ()
 {
    CLEAR (flags, ENDED);
    Game50 = 0;
-   GameCnt = -1;
+   RealGameCnt = GameCnt = -1;
    Game[0].hashkey = HashKey;
    TTClear ();
    PTClear ();
