@@ -1,5 +1,5 @@
 /* GNU Chess 5.0 - move.c - make and unmake moves code
-   Copyright (c) 1999 Free Software Foundation, Inc.
+   Copyright (c) 1999-2002 Free Software Foundation, Inc.
 
    GNU Chess is based on the two research programs 
    Cobalt by Chua Kong-Sian and Gazebo by Stuart Cracraft.
@@ -33,7 +33,7 @@
 #include <ctype.h>
 #include "common.h"
 
-void MakeMove (short side, int *move)
+void MakeMove (int side, int *move)
 /**************************************************************************
  *
  *  To make a move on the board and update the various game information.
@@ -41,9 +41,9 @@ void MakeMove (short side, int *move)
  **************************************************************************/
 {
    BitBoard *a;
-   short f, t, fpiece, tpiece; 
-   short rookf, rookt, epsq, sq;
-   short xside;
+   int f, t, fpiece, tpiece; 
+   int rookf, rookt, epsq, sq;
+   int xside;
    GameRec *g;
 
    xside = 1^side;
@@ -261,7 +261,7 @@ void MakeMove (short side, int *move)
 
 
 
-void UnmakeMove (short side, int *move)
+void UnmakeMove (int side, int *move)
 /****************************************************************************
  *
  *  To unmake a move on the board and update the various game information.
@@ -271,9 +271,9 @@ void UnmakeMove (short side, int *move)
  ****************************************************************************/
 {
    BitBoard *a;
-   short f, t, fpiece, cpiece;   
-   short rookf, rookt, epsq;
-   short xside;
+   int f, t, fpiece, cpiece;   
+   int rookf, rookt, epsq;
+   int xside;
    GameRec *g;
 
    side = 1^side;
@@ -377,7 +377,7 @@ void UnmakeMove (short side, int *move)
 }
 
 
-void SANMove (int move, short ply)
+void SANMove (int move, int ply)
 /****************************************************************************
  *
  *  Convert the move to a SAN format.  GenMoves (ply) needs to be called
@@ -385,8 +385,9 @@ void SANMove (int move, short ply)
  *
  ****************************************************************************/
 {
-   short piece, side, ambiguous;
-   short f, t;
+   int side;
+   int piece, ambiguous;
+   int f, t;
    BitBoard b;
    leaf *node1;
    char *s;
@@ -509,7 +510,7 @@ leaf * ValidateMove (char *s)
 {
    short f, t, side, rank, file, fileto;
    short piece, kount;
-   char mvstr[64], *p;
+   char mvstr[MAXSTR], *p;
    BitBoard b;
    leaf *n1, *n2;
 
@@ -525,9 +526,9 @@ leaf * ValidateMove (char *s)
    p = mvstr;
    do
    {
-      if (*s != 'x' && *s != '+' && *s != '=')
+      if (*s != 'x' && *s != '+' && *s != '=' && !isspace(*s))
          *p++ = *s; 
-   } while (*s++ != '\0');
+   } while (*s++ != '\0' );
 
    /* Flush castles that check */
    if (mvstr[strlen(mvstr)-1] == '+' || mvstr[strlen(mvstr)-1] == '#' ||
@@ -698,7 +699,7 @@ leaf * ValidateMove (char *s)
 }
 
 
-leaf * IsInMoveList (short ply, short f, short t, char piece)
+leaf * IsInMoveList (int ply, int f, int t, char piece)
 /**************************************************************************
  *
  *  Checks to see if from and to square can be found in the movelist
@@ -710,7 +711,7 @@ leaf * IsInMoveList (short ply, short f, short t, char piece)
 
    for (node = TreePtr[ply]; node < TreePtr[ply + 1]; node++)
    {
-      if ((short) (node->move & 0x0FFF) == MOVE(f,t)  && 
+      if ((int) (node->move & 0x0FFF) == MOVE(f,t)  && 
 	toupper(piece) == notation[PROMOTEPIECE (node->move)])
          return (node);
    }
@@ -718,7 +719,7 @@ leaf * IsInMoveList (short ply, short f, short t, char piece)
 }
 
 
-short IsLegalMove (int move)
+int IsLegalMove (int move)
 /*****************************************************************************
  *
  *  Check that a move is legal on the current board.  
@@ -733,7 +734,8 @@ short IsLegalMove (int move)
  *
  *****************************************************************************/
 {
-   short f, t, piece, side;
+   int side;
+   int f, t, piece;
    BitBoard blocker, enemy;
 
    f = FROMSQ(move); 
@@ -830,8 +832,6 @@ short IsLegalMove (int move)
    return (true);
 }
 
-
-static char s[6];
 char *AlgbrMove (int move)
 /*****************************************************************************
  *
@@ -839,7 +839,8 @@ char *AlgbrMove (int move)
  *
  *****************************************************************************/
 {
-   short f, t;
+   int f, t;
+   static char s[6];
 
    f = FROMSQ(move);
    t = TOSQ(move);
