@@ -1,5 +1,5 @@
 /* GNU Chess 5.0 - search.c - tree-search code
-   Copyright (c) 1999 Free Software Foundation, Inc.
+   Copyright (c) 1999-2002 Free Software Foundation, Inc.
 
    GNU Chess is based on the two research programs 
    Cobalt by Chua Kong-Sian and Gazebo by Stuart Cracraft.
@@ -23,9 +23,6 @@
      bug-gnu-chess@gnu.org
      cracraft@ai.mit.edu, cracraft@stanfordalumni.org, cracraft@earthlink.net
 */
-/*
- *
- */
 
 #include <stdio.h>
 #include <stdlib.h>
@@ -40,25 +37,21 @@
 #define FUTSCORE        (MATERIAL+fdel)
 #define GETNEXTMOVE  (InChk[ply] ? PhasePick1 (&p, ply) : PhasePick (&p, ply))
 
-inline void ShowThinking (leaf *, short);
-inline void ShowThinking (leaf *p, short ply)
+static inline void ShowThinking (leaf *p, uint8_t ply)
 {
    if (flags & XBOARD)
       return;
    if (!(flags & POST))
       return;
    if (NodeCnt < 500000 && (flags & SOLVE)) {
-      /* printf("NodeCnt = %d\n",NodeCnt); getchar(); */
       return;
    }
    SANMove (p->move, ply);
-   fprintf (stderr, "\r%2d.         %2d/%2d%10s    ", Idepth/DEPTH, 
+   printf ("\r%2d.         %2d/%2d%10s    ", Idepth/DEPTH, 
       (int) (p-TreePtr[ply]+1), (int) (TreePtr[ply+1]-TreePtr[ply]), SANmv);
-   fflush (stderr);
+   fflush (stdout);
 }
 
-static const short rank7[2] = { 6, 1 };
-static const short rank6[2] = { 5, 2 };
 static int ply1score;
 
 int SearchRoot (short depth, int alpha, int beta)
@@ -72,7 +65,8 @@ int SearchRoot (short depth, int alpha, int beta)
  **************************************************************************/
 {
    int best, score, savealpha;
-   short side, xside, ply, nodetype;
+   uint8_t side, xside, ply;
+   short nodetype;
    leaf *p, *pbest;
 
    ply = 1; 
@@ -152,17 +146,25 @@ int SearchRoot (short depth, int alpha, int beta)
       if (flags & TIMEOUT)
       {
 	/* XXX: It seems that ply == 1 always at this point */
+         ASSERT( ply == 1);
          best = (ply & 1 ? rootscore : -rootscore );
 	 return (best);
       }
 
-      if (SearchDepth == 0 && (NodeCnt & TIMECHECK) == 0)
+      if (((flags & PONDER) || SearchDepth == 0) && (NodeCnt & TIMECHECK) == 0)
       {
-         GetElapsed ();
-         if ((et >= SearchTime && (rootscore == -INFINITY-1 || 
-		ply1score > lastrootscore - 25 || flags & SOLVE)) ||
-	     et >= maxtime)
-	    SET (flags, TIMEOUT);        
+	 if (flags & PONDER) {
+	    if (input_status != INPUT_NONE)
+	       SET(flags, TIMEOUT);
+	 } else {
+	    ElapsedTime = GetElapsed (StartTime);
+	    if ((ElapsedTime >= SearchTime && (
+		    rootscore == -INFINITY-1 
+		    || ply1score > lastrootscore - 25 
+		    || flags & SOLVE))
+		|| ElapsedTime >= maxtime)
+	       SET (flags, TIMEOUT);
+	 }
       }
 
       if (MATE+1 == best+1)
@@ -189,7 +191,7 @@ done:
 }
 
 
-int Search (short ply, short depth, int alpha, int beta, short nodetype)
+int Search (uint8_t ply, short depth, int alpha, int beta, short nodetype)
 /**************************************************************************
  *
  *  The basic algorithm for this search routine came from Anthony 
@@ -200,8 +202,9 @@ int Search (short ply, short depth, int alpha, int beta, short nodetype)
  **************************************************************************/
 {
    int best, score, nullscore, savealpha;
-   short side, xside, rc, t0, t1, firstmove;
-   short fcut, fdel, donull, savenode, nullthreatdone, extend;
+   int side, xside;
+   int rc, t0, t1, firstmove;
+   int fcut, fdel, donull, savenode, nullthreatdone, extend;
    leaf *p, *pbest;
    int g0, g1;
    int upperbound;
@@ -279,7 +282,7 @@ int Search (short ply, short depth, int alpha, int beta, short nodetype)
    else if (g0 & PROMOTION)
    {
       PawnExtCnt++;
-      depth += DEPTH;
+      depth += DEPTH; /* Not reached, but why?! */
       extend = true;
    }
    /* Recapture extension */
@@ -334,14 +337,14 @@ int Search (short ply, short depth, int alpha, int beta, short nodetype)
    upperbound = INFINITY;
    if (flags & USEHASH)
    {
-      rc = TTGet (side, depth, ply, alpha, beta, &score, &g1);
+      rc = TTGet (side, depth, ply, &score, &g1);
       if (rc)
       {
          Hashmv[ply] = g1 & MOVEMASK;
          switch (rc)
          {
-            case POORDRAFT  :  break;
-	    case EXACTSCORE :  return (score);
+	 case POORDRAFT  :  /* Not reached */ break;
+	 case EXACTSCORE :  /* Not reached */ return (score);
             case UPPERBOUND :  beta = MIN (beta, score);
 			       upperbound = score;
 			       donull = false;
@@ -536,6 +539,7 @@ int Search (short ply, short depth, int alpha, int beta, short nodetype)
 	    }
          }
       }
+
       UnmakeMove (xside, &p->move);
 
 /*  Perform threat extensions code */
@@ -548,6 +552,7 @@ int Search (short ply, short depth, int alpha, int beta, short nodetype)
 	    threatply = ply;
 	    threatmv  = p->move;
             MakeNullMove (side);
+
             nullscore = -Search(ply+1, depth-1-R, -alpha+THREATMARGIN, 
 			-alpha+THREATMARGIN+1, nodetype);
             UnmakeNullMove (xside); 
@@ -580,13 +585,19 @@ int Search (short ply, short depth, int alpha, int beta, short nodetype)
 	 return (best);
       }
 
-      if (SearchDepth == 0 && (NodeCnt & TIMECHECK) == 0)
-      {
-         GetElapsed ();
-         if ((et >= SearchTime && (rootscore == -INFINITY-1 || 
-		ply1score > lastrootscore - 25 || flags & SOLVE)) ||
-	     et >= maxtime)
-	    SET (flags, TIMEOUT);        
+      if (((flags & PONDER) || SearchDepth == 0) && (NodeCnt & TIMECHECK) == 0)
+      {	
+	 if (flags & PONDER) {
+	    if (input_status != INPUT_NONE)
+	       SET(flags, TIMEOUT);
+	 } else {
+	    ElapsedTime = GetElapsed (StartTime);
+	    if ((ElapsedTime >= SearchTime && 
+		 (rootscore == -INFINITY-1 || 
+		  ply1score > lastrootscore - 25 || flags & SOLVE)) ||
+		ElapsedTime >= maxtime)
+	       SET (flags, TIMEOUT);        
+	 }
       }
 
 /*  The following line should be explained as I occasionally forget too :) */
@@ -609,8 +620,13 @@ done:
 */
 
    /*  Save the best move inside the transposition table  */
-   if (flags & USEHASH)
-      TTPut (side, depth, ply, savealpha, beta, best, pbest->move); 
+   if (flags & USEHASH){
+// Nasty temporary hack to try and work around timeout problem
+// If we are pondering and timeout don't save incomplete answers
+// Must look at failure of TIMEOUT condition more carefully!
+	if ( !(flags & TIMEOUT))
+          TTPut (side, depth, ply, savealpha, beta, best, pbest->move); 
+      }
 
    /*  Update history  */
    if (best > savealpha)
@@ -639,7 +655,7 @@ void ShowLine (int move __attribute__ ((unused)), int score, char c)
  *
  *****************************************************************************/
 {
-   short i, len;
+   int i, len;
    int pvar[MAXPLYDEPTH];
 
    /* SMC */
@@ -655,49 +671,59 @@ void ShowLine (int move __attribute__ ((unused)), int score, char c)
       return;
    if (rootscore == -INFINITY-1)
       return;
-   GetElapsed ();
+   ElapsedTime = GetElapsed (StartTime);
 
    /*
-    * What is the reason for these different output formats, in
-    * particular for et?
+    * The different output formats for Xboard and GNU Chess are documented
+    * in the engine protocol guide.
+    *
+    * In particular if the character after ply is not a space, Xboard
+    * assume it is talking to a GNU Chess compatible engine and
+    * uses time in seconds, not centiseconds.
+    *
+    * This code should be simplified!
+    *
     */
    if (flags & XBOARD) {
      if (score > MATE-255) {
        printf ("%d%c Mat%d %d %ld\t", Idepth/DEPTH, c,
-                (int)(MATE+2-abs(score))/2, (int)(et*100), NodeCnt+QuiesCnt);
+                (int)(MATE+2-abs(score))/2, (int)(ElapsedTime), NodeCnt+QuiesCnt);
        if (ofp != stdout)
-	 fprintf (ofp,"%2d%c%7.2f  Mat%02d%10ld\t", Idepth/DEPTH, c, et,
+	 fprintf (ofp,"%2d%c%7.2f  Mat%02d%10ld\t", Idepth/DEPTH, c, ElapsedTime,
                 (MATE+2-abs(score))/2, NodeCnt+QuiesCnt);
      } else if (score < -MATE+255) {
        printf ("%d%c -Mat%2d %d %ld\t", Idepth/DEPTH, c,
-                (int)(MATE+2-abs(score))/2, (int)(et*100), NodeCnt+QuiesCnt);
+                (int)(MATE+2-abs(score))/2, (int)(ElapsedTime), NodeCnt+QuiesCnt);
        if (ofp != stdout)
-	 fprintf (ofp,"%2d%c%7.2f -Mat%02d%10ld\t", Idepth/DEPTH, c, et,
+       fprintf (ofp,"%2d%c%7.2f -Mat%02d%10ld\t", Idepth/DEPTH, c, ElapsedTime,
 		 (MATE+2-abs(score))/2, NodeCnt+QuiesCnt);
      } else {
 	 printf ("%d%c %d %d %ld\t", Idepth/DEPTH, c, (int)score, 
-		 (int)(et*100), NodeCnt+QuiesCnt);
+		 (int)(ElapsedTime), NodeCnt+QuiesCnt);
 	 if (ofp != stdout) 
-	   fprintf (ofp,"%2d%c%7.2f%7d%10ld\t", Idepth/DEPTH, c, et, score, NodeCnt+QuiesCnt);	 
+	   fprintf (ofp,"%2d%c%7.2f%7d%10ld\t", Idepth/DEPTH, c, 
+		    ElapsedTime, score, NodeCnt+QuiesCnt);	 
        }
    }
-   else {
-     if (score > MATE-255) {
-       printf ("\r%2d%c%7.2f  Mat%02d%10ld\t", Idepth/DEPTH, c, et,
-                (MATE+2-abs(score))/2, NodeCnt+QuiesCnt);
-       if (ofp != stdout)
-	 fprintf (ofp,"\r%2d%c%7.2f  Mat%02d%10ld\t", Idepth/DEPTH, c, et,
-                (MATE+2-abs(score))/2, NodeCnt+QuiesCnt);
-     } else if (score < -MATE+255) {
-       printf ("\r%2d%c%7.2f -Mat%02d%10ld\t", Idepth/DEPTH, c, et,
-	     (MATE+2-abs(score))/2, NodeCnt+QuiesCnt);
-       if (ofp != stdout)
-	 fprintf (ofp,"\r%2d%c%7.2f -Mat%02d%10ld\t", Idepth/DEPTH, c, et,
+   else {              // Not XBOARD
+      if (score > MATE-255) {
+	 printf ("\r%2d%c%7.2f  Mat%02d%10ld\t", Idepth/DEPTH, c, ElapsedTime,
 		 (MATE+2-abs(score))/2, NodeCnt+QuiesCnt);
-     } else {
-	 printf ("\r%2d%c%7.2f%7d%10ld\t", Idepth/DEPTH, c, et, score, NodeCnt+QuiesCnt);
+	 if (ofp != stdout)
+	    fprintf (ofp,"\r%2d%c%7.2f  Mat%02d%10ld\t", Idepth/DEPTH, c, ElapsedTime,
+		     (MATE+2-abs(score))/2, NodeCnt+QuiesCnt);
+      } else if (score < -MATE+255) {
+	 printf ("\r%2d%c%7.2f -Mat%02d%10ld\t", Idepth/DEPTH, c, ElapsedTime,
+		 (MATE+2-abs(score))/2, NodeCnt+QuiesCnt);
+	 if (ofp != stdout)
+	    fprintf (ofp,"\r%2d%c%7.2f -Mat%02d%10ld\t", Idepth/DEPTH, c, ElapsedTime,
+		     (MATE+2-abs(score))/2, NodeCnt+QuiesCnt);
+      } else {
+	 printf ("\r%2d%c%7.2f%7d%10ld\t", Idepth/DEPTH, c, ElapsedTime,
+		 score, NodeCnt+QuiesCnt);
 	 if (ofp != stdout) 
-	   fprintf (ofp,"\r%2d%c%7.2f%7d%10ld\t", Idepth/DEPTH, c, et, score, NodeCnt+QuiesCnt);	 
+	    fprintf (ofp,"\r%2d%c%7.2f%7d%10ld\t", Idepth/DEPTH, c, ElapsedTime,
+		    score, NodeCnt+QuiesCnt);	 
       }
    }
 
